==> ts-pegjs-2.1.0.tgz: package/README.md <==
# TS PEG.js

TS PEG.js is a TS code generation plugin for [peggy](https://www.npmjs.com/package/peggy).

[![Build Status](https://travis-ci.org/metadevpro/ts-pegjs.svg?branch=master)](https://travis-ci.org/metadevpro/ts-pegjs)
[![Known Vulnerabilities](https://snyk.io/test/github/metadevpro/ts-pegjs/badge.svg)](https://snyk.io/test/github/metadevpro/ts-pegjs)
[![npm version](https://badge.fury.io/js/ts-pegjs.svg)](http://badge.fury.io/js/ts-pegjs)

[![NPM](https://nodei.co/npm/ts-pegjs.png?downloads=true&downloadRank=true&stars=true)](https://nodei.co/npm/ts-pegjs/)

## Requirements

-   [peggy](https://www.npmjs.com/package/peggy) (previous versions use: [pegjs](https://pegjs.org))

## Installation

### Node.js

Installs ts-pegjs + peggy

    $ npm install ts-pegjs

## Usage

### Generating a Parser from JS code

In Node.js, require both the peggy parser generator and the ts-pegjs plugin:

```typescript
var peggy = require('peggy');
var tspegjs = require('ts-pegjs');
```

To generate a TS parser, pass to `pegjs.generate` ts-pegjs plugin and your grammar:

```typescript
var parser = pegjs.generate("start = ('a' / 'b')+", {
    output: 'source',
    format: 'commonjs',
    plugins: [tspegjs],
    tspegjs: {
        customHeader: "// import lib\nimport { Lib } from 'mylib';"
    }
});
```

The method will return source code of generated parser as a string.

Supported options of `pegjs.generate`:

-   `cache` — if `true`, makes the parser cache results, avoiding exponential
    parsing time in pathological cases but making the parser slower (default:
    `false`). This is strongly recommended for big grammars
    (like javascript.pegjs or css.pegjs in example folder)
-   `allowedStartRules` — rules the parser will be allowed to start parsing from
    (default: the first rule in the grammar)

### Plugin options

-   `custom-header` — A string or an array of strings which are a valid TS code to be injected on the header of the output file. E.g. provides a convenient place for adding library imports. 
-   `returnTypes` — An object containing rule names as keys and a valid TS return type as string.

### Generating a Parser from CLI

Sample usage:

```
peggy --plugin ./src/tspegjs -o examples/arithmetics.ts --cache examples/arithmetics.pegjs
```

It will generarate the parser in the TS flavour.

If you need to pass specific plugin options you can use the option `--extra-options-file` provided by pegjs and pass it a filename (e.g. pegconfig.json) containing specific options like the following JSON sample:

```
peggy --plugin ./src/tspegjs --extra-options-file pegconfig.json -o examples/arithmetics.ts --cache examples/arithmetics.pegjs
```

```json
{
    "tspegjs": {
        "customHeader": "// import lib\nimport { Lib } from 'mylib';"
    },
    "returnTypes": {
        "Integer": "number",
        "Expression": "number",
    }
}
```
> For rules not listed in `returnTypes` object `any` type is declared by default.

> Make sure to pass any additional CLI options, like `--extra-options-file` before the parameter `-o` as these will otherwise be treated as arguments to that one.

## Using the Parser

1. Save parser generated by `pegjs.generate` to a file or use the one generated from the CLI tool.

2. In client TS code:

```typescript
import { SyntaxError, parse } from './arithmetics';

try {
    const sampleOutput = parse('my sample...');
} catch (ex: SyntaxError) {
    // Handle parsing error
    // [...]
}
```

## Changelog

[Changelog](./Changelog.md).

## Acknowledgments

Thanks to:

-   [David Majda](https://github.com/dmajda) for creating pegjs
-   [Elantcev Mikhail](https://github.com/Nordth) for providing the pegjs PHP plugin, inspiration on this one.

## License

[The MIT License (MIT)](http://opensource.org/licenses/MIT)

---

(c) 2017-2022, [Pedro J. Molina](https://github.com/pjmolina) at [metadev.pro](https://metadev.pro)

==> ts-pegjs-2.2.1.tgz: package/README.md <==
# TS PEG.js

TS PEG.js is a TS code generation plugin for [peggy](https://www.npmjs.com/package/peggy).

[![Build Status](https://travis-ci.org/metadevpro/ts-pegjs.svg?branch=master)](https://travis-ci.org/metadevpro/ts-pegjs)
[![Known Vulnerabilities](https://snyk.io/test/github/metadevpro/ts-pegjs/badge.svg)](https://snyk.io/test/github/metadevpro/ts-pegjs)
[![npm version](https://badge.fury.io/js/ts-pegjs.svg)](http://badge.fury.io/js/ts-pegjs)

[![NPM](https://nodei.co/npm/ts-pegjs.png?downloads=true&downloadRank=true&stars=true)](https://nodei.co/npm/ts-pegjs/)

## Requirements

-   [peggy](https://www.npmjs.com/package/peggy) (previous versions use: [pegjs](https://pegjs.org))

## Installation

### Node.js

Installs ts-pegjs + peggy

    $ npm install ts-pegjs

## Usage

### Generating a Parser from JS code

In Node.js, require both the peggy parser generator and the ts-pegjs plugin:

```typescript
var peggy = require('peggy');
var tspegjs = require('ts-pegjs');
```

To generate a TS parser, pass to `pegjs.generate` ts-pegjs plugin and your grammar:

```typescript
var parser = pegjs.generate("start = ('a' / 'b')+", {
    output: 'source',
    format: 'commonjs',
    plugins: [tspegjs],
    tspegjs: {
        customHeader: "// import lib\nimport { Lib } from 'mylib';"
    }
});
```

The method will return source code of generated parser as a string.

Supported options of `pegjs.generate`:

-   `cache` — if `true`, makes the parser cache results, avoiding exponential
    parsing time in pathological cases but making the parser slower (default:
    `false`). This is strongly recommended for big grammars
    (like javascript.pegjs or css.pegjs in example folder)
-   `allowedStartRules` — rules the parser will be allowed to start parsing from
    (default: the first rule in the grammar)

### Plugin options

**Note:** Options in CLI mode are written in POSIX (long names as kebab-case) convention e.g. `--custom-header` but with camelcase on JavaScript e.g. `customHeader`.

-   `customHeader` — A string or an array of strings which are a valid TS code to be injected on the header of the output file. E.g. provides a convenient place for adding library imports.
-   `customHeaderFile` — A header file to include.
-   `errorName` — The name of the exported internal error class to override. For backward compatibility the default value is `SyntaxError`.
-   `returnTypes` — An object containing rule names as keys and a valid TS return type as string.

### Generating a Parser from CLI

Sample usage:

```
peggy --plugin ./src/tspegjs -o examples/arithmetics.ts --cache examples/arithmetics.pegjs
```

(Note `./src/tspegjs` is the path to `tspegjs.js` in the project. If you installed ts-pegjs using npm, it should probably be `./node_modules/ts-pegjs/src/tspegjs`.)

It will generarate the parser in the TS flavour.

If you need to pass specific plugin options you can use the option `--extra-options-file` provided by pegjs and pass it a filename (e.g. pegconfig.json) containing specific options like the following JSON sample:

```
peggy --plugin ./src/tspegjs --extra-options-file pegconfig.json -o examples/arithmetics.ts --cache examples/arithmetics.pegjs
```

```json
{
    "tspegjs": {
        "customHeader": "// import lib\nimport { Lib } from 'mylib';"
    },
    "returnTypes": {
        "Integer": "number",
        "Expression": "number",
    }
}
```
> For rules not listed in `returnTypes` object `any` type is declared by default.

> Make sure to pass any additional CLI options, like `--extra-options-file` before the parameter `-o` as these will otherwise be treated as arguments to that one.

## Using the Parser

1. Save parser generated by `pegjs.generate` to a file or use the one generated from the CLI tool.

2. In client TS code:

```typescript
import { SyntaxError, parse } from './arithmetics';

try {
    const sampleOutput = parse('my sample...');
} catch (ex: SyntaxError) {
    // Handle parsing error
    // [...]
}
```

## Changelog

[Changelog](./Changelog.md).

## Acknowledgments

Thanks to:

-   [David Majda](https://github.com/dmajda) for creating pegjs
-   [Elantcev Mikhail](https://github.com/Nordth) for providing the pegjs PHP plugin, inspiration on this one.

## License

[The MIT License (MIT)](http://opensource.org/licenses/MIT)

---

(c) 2017-2022, [Pedro J. Molina](https://github.com/pjmolina) at [metadev.pro](https://metadev.pro)
